// pvsynthesizer.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993, 1994 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/
#ifndef PVSYNTHESIZER_H
#ifdef __GNUG__
#pragma interface
#endif
#define PVSYNTHESIZER_H

#include "localdefs.h"
#include "outputfunction.h"
#include "range.h"

class PhaseVocoder;
class PvocData;

class PVSynthesizer : public QueuedOutputFunction {
	typedef QueuedOutputFunction Super;
public:
	PVSynthesizer(Data* output, PvocData* anal, int L, int I,
		double T, int i, int j, boolean useKaiser);
	PVSynthesizer(Data* output, PvocData* anal);
	static Modifier* create(class DataEditor*);
	virtual ~PVSynthesizer();
	redefined void restoreState();
	redefined const char* message() {
		return "Running phase vocoder synthesis...";
	}
protected:
	redefined Requester* createRequester();
	redefined void initialize();
	redefined int doProcessing();
private:
	PhaseVocoder::Info pvocInfo;
	PhaseVocoder* pvoc;
	PvocData* pvdata;
	Range currentFrame;
};

#endif
