// wavetable.h

/******************************************************************************
 *
 *  MiXViews - an X window system based sound & data editor/processor
 *
 *  Copyright (c) 1993, 1994 Regents of the University of California
 *
 *  Author:     Douglas Scott
 *  Date:       December 13, 1994
 *
 *  Permission to use, copy and modify this software and its documentation
 *  for research and/or educational purposes and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation. The author reserves the right to distribute this
 *  software and its documentation.  The University of California and the author
 *  make no representations about the suitability of this software for any 
 *  purpose, and in no event shall University of California be liable for any
 *  damage, loss of data, or profits resulting from its use.
 *  It is provided "as is" without express or implied warranty.
 *
 ******************************************************************************/


// Encapsulation of a simple interpolated lookup table, for storage of
// waveforms and windowing functions.  Used in Buzz and FFT_Function classes.

#ifndef WAVETABLE_H
#ifdef __GNUG__
#pragma interface
#endif
#define WAVETABLE_H

#include <InterViews/resource.h>

class WaveTable : public Resource {
public:
	WaveTable(int size=1024);
	virtual ~WaveTable();
	virtual double operator [] (int index);
	virtual double operator [] (double index);
	int size() { return mySize; }
protected:
	void zeroTable();
	int wrapIndex(int i) {
		while(i >= mySize) i -= mySize; return i;
	}
	double wrapIndex(double i) {
		while(i >= double(mySize)) i -= mySize; return i;
	}
protected:
	double* table;
	int mySize;
};

class SineTable : public WaveTable {
public:
	SineTable(int size=1024);
	virtual ~SineTable() {}
};

#endif
